'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Transitioned,
 *  wrapper component, used to transition a prop from a previous state to a next,
 *    which is useful to use combined to css transition/animation
 *
 *  usage is similar to react-motion
 *
 *  use as :
 *
 *  ...<div>
 *       <Transitioned toTransition={ this.props.a } delay={ delay }  >
 *         {
 *           ({ next, previous }) =>  <div> was { previous } will be { next } </div>
 *         }
 *       </Transitioned>
 *    <div>
 *
 *  Transitioned children should be a unique function. This function receive in argument next and previous, which are the value to transition
 *
 *  Flow diagram :
 *
 *    k = 'a'             k change to 'b'
 *                             |
 *                             |<------------delay---------------->|
 *    . . . ------------------------------------------------------------------------------>
 *        next     : a                 next     : b                    next     : b
 *        previous : null              previous : a                    previous : null
 *
 *        transition : false           transition : true               transition : false
 *
 *
 *
 */

var Transitioned = function (_React$Component) {
  _inherits(Transitioned, _React$Component);

  function Transitioned(props) {
    _classCallCheck(this, Transitioned);

    var _this = _possibleConstructorReturn(this, (Transitioned.__proto__ || Object.getPrototypeOf(Transitioned)).call(this, props));

    _this.state = { transition: false, next: _this.props.toTransition };
    _this.cancel = null;

    _this.fadeOff = _this.fadeOff.bind(_this);
    return _this;
  }

  _createClass(Transitioned, [{
    key: 'fadeOff',
    value: function fadeOff() {
      clearTimeout(this.cancel);

      this.setState({ previous: null, transition: false });
    }
  }, {
    key: '_equal',
    value: function _equal(a, b) {
      return this.props.equal ? !a === !b && (!a || this.props.equal(a, b)) : a === b;
    }
  }, {
    key: '_delay',
    value: function _delay() {
      return this.props.delay || 5000;
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var next = nextProps.toTransition;
      var previous = this.state.next;

      if (!this._equal(previous, next)) {
        this.setState({ next: next, previous: previous, transition: true });

        clearTimeout(this.cancel);
        this.cancel = setTimeout(this.fadeOff, this._delay());
      } else {
        this.setState({ next: next });
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      clearTimeout(this.cancel);
    }
  }, {
    key: 'render',
    value: function render() {
      var renderChildren = this.props.children[0] || this.props.children;
      var renderedChildren = renderChildren(this.state);
      return renderedChildren && _react2.default.Children.only(renderedChildren) || null;
    }
  }]);

  return Transitioned;
}(_react2.default.Component);

Transitioned.propTypes = {
  toTransition: _propTypes.PropTypes.any,
  equal: _propTypes.PropTypes.func,
  delay: _propTypes.PropTypes.number
};

exports.default = Transitioned;